/**
 * ==================================================================
 *
 *
 * Program Name
 *  PUSHER - Message Pusher
 *
 * Last date of modification
 *  January 10, 2017
 *
 * Description
 *  This program will destructively get a message from a queue and
 *  output it to another queue under syncpoint.  Context information
 *  is maintained across the message put.  Basically, this program is a
 *  message pusher.
 *
 *  The following is the logic in this program:
 *
 *  - Initialize and allocated memory.
 *  - Get the command line parameters.
 *  - Read the ini file.
 *  - Connect to the queue manager.
 *  - Open both the input queue and the output queue.
 *  - Read the first Message under syncpoint
 *  - While more messages
 *  -   Output the message under syncpoint
 *  -   Commit messages
 *  -   Read the next message under syncpoint
 *  - EndWhile
 *  - Close the input queue
 *  - Close output queue at index
 *  - EndLoop
 *  - Disconnect from the queue manager.
 *  - Output statistical information.
 *  - Terminate
 *
 * @copyright Copyright(C), 1996-2017, Capitalware Inc.
 * @author Roger Lacroix, Capitalware Inc.
 * @return 0 for ok or -1 for error
 * @version 1.0.0
 * @license Apache 2 License
 *
 * ------------------------------------------------------------------
 */
#if defined(__MVS__)
#define MVS /* define our MVS indicator */
#define _XOPEN_SOURCE_EXTENDED 1
#endif

/**
 * Program build time eye-catchers for executable.
 */
#define PROGRAM_NAME "PUSHER"
static const char * EYECATCHER_EyeCatcher  = PROGRAM_NAME "  ";
static const char * EYECATCHER_Version     = "1.0.0" ;
static const char * EYECATCHER_Datestamp   = __DATE__ ;
static const char * EYECATCHER_Timestamp   = __TIME__ ;
static const char * EYECATCHER_Filename    = __FILE__ ;

/**
 * Include the C standard header files
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

/**
 * Include MQSeries header files
 */
#include <cmqc.h>

/**
 * PUSHER include file.
 */
#if defined(MVS) || defined(OS400)
#include "pusher.h"
#else
#include "hdrs/pusher.h"
#endif


/**
 * Global Storage.
 */

char* usage = "-i ini_filename                                     \n"\
"-i ini_filename    The name of the ini file                       \n"\
"-?                 Display this help message            [Optional]\n";

/**
 * PUSHER's global work storage.
 */
PUSHER_GLOBAL_T  MG;

/**
 * Mainline.
 */
int main( int argc, char **argv )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int          rc=0;
   int          rc2=0;
   int          i;

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   rc = Initialize( );

   if ( rc == 0 )
   {
      if (argc == 1)
      {
         /* no parameters specified!! */
      }
      else if ( (argc == 2) &&
                ( memcmp(argv[1], MQTMC_STRUC_ID, 4) == STR_EQ ) )
      {
         /* Get any trigger related parameters */
         rc = GetTriggerParms( argc, argv );
      }
      else if (argc == 3)
      {
         if ( memcmp(argv[1], MQTMC_STRUC_ID, 4) == STR_EQ )
         {
            /* Get any trigger related parameters */
            rc = GetTriggerParms( argc, argv );
         }
         else
         {
            /* Get the command line parameters */
            rc = GetCmdLineParms( argc, argv );
         }
      }
      else
      {
         printf( "\nInvalid number of parameters!\n\n" );

         printf( "argc=%d\n", argc);

         for(i=0; i<argc; i++)
            printf( "argv[%d]=%s\n", i, argv[i]);

         printf( "%s %s\n", argv[0], usage);
         rc = 12;
      }
   }

   if ( rc == 0 )
   {
      /* Read ini file */
     rc = GetInitFileParameters( );
   }

   if ( rc == 0 )
   {
      /* Connect to the local queue manager */
      rc = ConnectToMqm( );
   }

   if ( rc == 0 )
   {
      /* Open the Input Q */
      rc = OpenInputQueue( );
   }

   if ( rc == 0 )
   {
      /* Open the Output Q */
      rc = OpenOutputQueue();
   }

   if ( rc == 0 )
   {
      /* Loop the messages */
      rc = ProcessMessages( );
   }

   if ( MG.OpenedInputQueue == 1 )
   {
      /* We're done - close the Input Q */
      rc2 = CloseInputQueue( );

      if ( rc2 > rc )
      {
         rc = rc2;
      }
   }

   /* and also close the Output Q */
   if ( MG.OpenedOutputQueue == 1 )
   {
      rc2 = CloseOutputQueue();

      if ( rc2 > rc )
      {
         rc = rc2;
      }
   }


   /* if we are connected then disconnect */
   if ( MG.ConnectedFlag == 1 )
   {
      rc2 = DisconnectFromMqm( );

      if ( rc2 > rc )
         rc = rc2;
   }

   /* Tell the user what we did. */
   PrintStatistics( );

   /* Time to clean up. */
   Terminate( );

   return(0);
}


/**
 * ==================================================================
 *
 * Function Name
 *  Initialize
 *
 * Description
 *  This function will initialize the various fields.
 *
 * Input parameters
 *  None.
 *
 * Output
 *  MG  - clear global structure
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  Initialize( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int               rc=0;

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering Initialize\n")

   /**
    * Initialize the rest of the global variables.
    */
   memset(&MG, '\0', sizeof(PUSHER_GLOBAL_T));
#if defined(MVS)
   strcpy( MG.iniFile, "DD:PUSHERIN" );
#elif defined(OS400)
   strcpy( MG.iniFile, "QIBM/UserData/capitalware/PUSHER/pusher.ini" );
#else
   strcpy( MG.iniFile, "pusher.ini" );
#endif

   return rc;
}

/**
 * ==================================================================
 *
 * Function Name
 *  Terminate
 *
 * Description
 *  This function will free or de-allocate various fields.
 *
 * Input parameters
 *  None.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void Terminate( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering Terminate\n")

   if ( MG.MessageBuffer != NULL )
      free( MG.MessageBuffer );

   return;
}


/**
 * ==================================================================
 *
 * Function Name
 *   GetTriggerParms
 *
 * Description
 *  This function will attempt to get the iniFile name from the
 *  TriggerData field.
 *
 * Input parameters
 *  None.
 *
 * Output
 *  MG.iniFile                - set to default value
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  GetTriggerParms( int argc, char *argv[] )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int             rc=0,len=0;
   char            Buffer[101];
   PMQTMC2         pTMC;

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering GetTriggerParms\n")

   pTMC = (PMQTMC2) argv[1];

   if ((pTMC->TriggerData[0] != '\0') && (pTMC->TriggerData[0] != ' '))
   {
      len = sizeof(pTMC->TriggerData);
      memcpy( Buffer, pTMC->TriggerData, len );
      Buffer[len] = '\0';
      RemoveTrailingBlanks( Buffer );

      strncpy( MG.iniFile, Buffer, _MAX_PATH);

      TEST_DATA(("GetTriggerParms MG.iniFile=%s",MG.iniFile))
   }

   return rc;
}


/**
 * ==================================================================
 *
 * Function Name
 *  GetCmdLineParms
 *
 * Description
 *  This function will parse the inputted command line and
 *  extract the keyswords and their values.
 *
 * Input parameters
 *  argc - The number of parameters
 *  argv - An array of keywords and values.
 *
 * Output
 *  MG.iniFile    - set ini filename
 *  MG.LogFile    - set log filename
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  GetCmdLineParms( int argc, char *argv[] )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   #define MaxKeyParams 3

   char     *Parmkey[MaxKeyParams] = { "-i", "-I", "-?"};
   int      i,j;

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering GetCmdLineParms\n")

   if (argc > 0)
   {
      for (i=1; i < argc; i++)
      {
         for (j=0; j < MaxKeyParams; j++)
         {
            if (!strncmp( argv[i], Parmkey[j], 2))
            {
                break; /* fall out to switch stmt */
            }
         }

         switch (j)
         {
            case 0:                      /* -i */
            case 1:                      /* -I */
              if (i+1 < argc)
              {
#if defined(MVS)
                 sprintf(MG.iniFile, "DD:%s", argv[i+1] );
#else
                 strncpy( MG.iniFile, argv[i+1], _MAX_PATH );
                 MG.iniFile[_MAX_PATH - 1] = '\0';
#endif
                 i++;
              }
              else
              {
                 printf( "Error with keyword -i value\n" );
                 printf( "Use %s -? for usage\n", argv[0]);
                 return 12;
              }
              break;

            case 2:                      /* -? */
               printf( "\n%s %s\n", argv[0], usage);
               return 12;
               break;

            default:/* parameter error */
               printf( "Error with keyword parameter!\n" );
               printf( "Invalid flag.  Use %s -? for usage\n",argv[0]);
               return 12;
               break;
         }
      }
   }
   else
   {
      printf( "\nMissing parameters!\n\n" );
      printf( "%s %s\n", argv[0], usage);
      return 12;
   }

   return 0;
}

/**
 * ==================================================================
 *
 * Function Name
 *  GetInitFileParameters
 *
 * Description
 *  This function reads an initialization file for input parameters.
 *
 * Input parameters
 *  MG   - Pointer to the Global PUSHER work area.
 *
 * Output
 *  MG.SequenceNumberFlag - Will the ini file contain sequence num
 *  MG.LogFile            - Path and name of the log file
 *  MG.QMgrName           - set field
 *  MG.InputQName         - set field
 *  MG.OutputQName1       - set field
 *  MG.OutputQName2       - set field
 *  MG.TimeOut            - Set MQGET timeout value
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  GetInitFileParameters( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int          rc = 0;
   int          Length;
   char         TempString[512];
   char         iniBuffer[MAX_INI_LINE_LENGTH];
   char         Buffer[3] = {'\0', '\0', '\0'};

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering GetInitFileParameters\n")

   TEST_DATA(("inifile=%s\n", MG.iniFile))

   /**
    * First get info about sequence numbers from ini file.
    */
   Length = GetProfileString( INI_SEQUENCE_NUMBER_FLAG,
                              DEFAULT_INI_NO,
                              TempString,
                              72,
                              MG.iniFile,
                              'Y' );

   strcpy( Buffer, TempString);   /* move flag */

   /**
    * Did we find it?  If not found, assume "N"
    */
   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_SEQUENCE_NUMBER_FLAG );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );

      MG.SequenceNumberFlag = 'N';
   }
   else if ( ( *(Buffer) == 'y' ) || ( *(Buffer) == 'Y' ) )
   {
      MG.SequenceNumberFlag = 'Y';  /* uppercase it */
   }
   else if ( ( *(Buffer) == 'n' ) || ( *(Buffer) == 'N' ) )
   {
      MG.SequenceNumberFlag = 'N';  /* lowercase it */
   }
   else
   {
      MG.SequenceNumberFlag = 'N'; /* everything else make it No */
   }

   /**
    * Retrieve logfile info from ini file.
    */
   Length = GetProfileString( INI_LOGFILE,
                              DEFAULT_LOGFILE,
                              iniBuffer,
                              sizeof(iniBuffer),
                              MG.iniFile,
                              MG.SequenceNumberFlag );

   if ( Length == 0 )
   {                                   /* not found, tell the world. */
      printf( "Error: LogFile was not found.\n" );
      printf( "Specify the LogFile name in the configuration file\n");
      printf( "Using default value: %s\n", DEFAULT_LOGFILE );

      /**
       * Set a default value for the logfile
       */
      #ifdef MVS
      sprintf( MG.LogFile, "DD:%s", DEFAULT_LOGFILE );
      #else
      strcpy( MG.LogFile, DEFAULT_LOGFILE );
      #endif
   }
   else
   {
      #ifdef MVS
      int  k;

      /**
       * Count the length of logfile name
       */
      for (k = 0; k < _MAX_PATH; k++)
      {
         if ( iniBuffer[k] == '\0' )
         {
            break;
         }
      }

      /**
       * Check the length of logfile name
       */
      if (k > 8)
      {
         printf( "Error: Log file DD '%s' is too long.\n",
                 iniBuffer );

         /**
          * We will use just the first 8 characters
          */
         sprintf( MG.LogFile, "DD:%8.8s", iniBuffer );
      }
      else
      {
         sprintf( MG.LogFile, "DD:%s", iniBuffer );
      }

      #else
      strcpy( MG.LogFile, iniBuffer );
      #endif
   }

   /**
    * Mark the beginning of the log file.
    */
   LogIt( LOG_TYPE_INFO,
          __LINE__,
          "********************************************");

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          "PUSHER is now starting.");

   sprintf(TempString,
           "%s version %s (%s)",
           PROGRAM_NAME,
           EYECATCHER_Version,
           HOSTED_PLATFORM_NAME);
   LogIt( LOG_TYPE_INFO,  __LINE__, TempString );

   sprintf( TempString, "Current Log file is %s", MG.LogFile );
   LogIt( LOG_TYPE_INFO,  __LINE__, TempString );

   /**
    * Get all integer / binary values from the ini file.
    */

   /**
    * Retrieve the Message Buffer Size
    */
   MG.MessageBufferSize = GetProfileInt( INI_MSG_BUFFER_SIZE,
                                          DEFAULT_MSG_BUFFER_SIZE,
                                          MG.iniFile,
                                          MG.SequenceNumberFlag );

   /**
    * Get all string values from the ini file.
    */

   /**
    * Retrieve the Queue Manager Name
    */
   Length = GetProfileString( INI_QMGR_NAME,
                              DEFAULT_INI_BLANK,
                              MG.QMgrName,
                              sizeof(MG.QMgrName),
                              MG.iniFile,
                              MG.SequenceNumberFlag );

   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_QMGR_NAME );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );
   }

   /**
    * Retrieve the Input Queue Name
    */
   Length = GetProfileString( INI_INPUT_Q_NAME,
                              DEFAULT_INI_BLANK,
                              MG.InputQName,
                              sizeof(MG.InputQName),
                              MG.iniFile,
                              MG.SequenceNumberFlag );

   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_INPUT_Q_NAME );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );
   }

   /**
    * Retrieve the Output Queue Name
    */
   Length = GetProfileString( INI_OUTPUT_Q_NAME,
                              DEFAULT_INI_BLANK,
                              MG.OutputQName,
                              sizeof(MG.OutputQName),
                              MG.iniFile,
                              MG.SequenceNumberFlag );

   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_OUTPUT_Q_NAME );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );
   }

   /**
    * Write all values to the log file??
    */
   Print_Global( );

   return rc;
}

/**
 * ==================================================================
 *
 * Function Name
 *  Print_Global
 *
 * Description
 *  This function will dump the global area.
 *
 * Input parameters
 *  None.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void Print_Global( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   char            Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering Print_Global\n")

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          "The following are the parameters used for this execution.");

   /* Output the Message Buffer Size */

   sprintf( MG.TempBuffer,
            "   Get Buffer Size   : %d",
            MG.MessageBufferSize );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MG.TempBuffer );

   /* Output Queue Manager name */

   memcpy( Buffer,
           MG.QMgrName,
           sizeof(MG.QMgrName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';

   RemoveTrailingBlanks( Buffer );

   sprintf( MG.TempBuffer,
            "   Queue Manager Name: %s",
            Buffer);

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MG.TempBuffer );

   /* Output the Input Message Q name */

   memcpy( Buffer,
           MG.InputQName,
           sizeof(MG.InputQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';

   RemoveTrailingBlanks( Buffer );

   sprintf( MG.TempBuffer,
            "   Input Queue Name  : %s",
            Buffer );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MG.TempBuffer );

   /* Output the number of Output Queues */


   sprintf( MG.TempBuffer,
            "   Output Queue Name  : %s",
            Buffer );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MG.TempBuffer );

   return;
}

/**
 * ==================================================================
 *
 * Function Name
 *  PrintStatistics
 *
 * Description
 *  This function will print statistic information about what was done.
 *
 * Input parameters
 *  None.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void PrintStatistics( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering PrintStatistics\n")

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          "Statistics of this executions:" );

   /* Output MQGET count */

   sprintf( MG.TempBuffer,
            "   Messages Read     : %d",
            MG.TotalMsgRead );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MG.TempBuffer );

   /* Output MQPUT count */

   sprintf( MG.TempBuffer,
            "   Messages Written  : %d",
            MG.TotalMsgWritten );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MG.TempBuffer );

   /* Tell the world we are going away. */

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          "PUSHER is now stopping.");

   return;
}

/**
 * ==================================================================
 *
 * Function Name
 *  LogIt
 *
 * Description
 *  This function will write a record to the logfile.
 *
 * Input parameters
 *  severity- The level of severity to be associated with this msg.
 *  line    - Line number of the calling function.
 *  msg     - The message text to be outputted.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void LogIt( int         severity,
            int         line,
            char       *msg )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   char            LogRecord[64];
   char            cSeverity[10];
   FILE           *LogFileHandle;
   struct tm*      CurrentTime;
   time_t          LongTime;

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
/*   TEST_PRINT("Now entering LogIt\n") */

   /**
    * Open up the log file
    */
   if ( *(MG.LogFile) == '\0' )
   {
      LogFileHandle = stdout;              /* write to the console */
   }
   else
   {
      LogFileHandle = fopen( MG.LogFile, "a" ); /* write to a file */

      if ( LogFileHandle == NULL )
      {
         printf( "Failed to open the log file.\n" );
         return;
      }
   }

   /**
    * Change severity level to text description.
    */
   switch( severity )
   {
      case LOG_TYPE_INFO:
       strcpy( cSeverity, "INFO   " );
       break;
      case 1:
       strcpy( cSeverity, "WARNING" );
       break;
      case 2:
       strcpy( cSeverity, "ERROR  " );
       break;
      default:
       strcpy( cSeverity, "ERROR+ " );
       break;
    }

   /**
    * Get the current (local) time.
    */
   LongTime = time(NULL);
   CurrentTime = localtime(&LongTime);

   sprintf( LogRecord,
            LOG_RECORD_FORMAT_STRING,
            (CurrentTime->tm_year + 1900),
            (CurrentTime->tm_mon + 1),
            CurrentTime->tm_mday,
            CurrentTime->tm_hour,
            CurrentTime->tm_min,
            CurrentTime->tm_sec,
            cSeverity,
            line );

   /* Write out the date and time */
   fputs( LogRecord, LogFileHandle );

   /* Write out the text  */
   fputs( msg, LogFileHandle );

   /* Put a new line character after the data. */
   fputs( "\n", LogFileHandle );
   fflush( LogFileHandle );

   if ( *(MG.LogFile) != '\0' )
      fclose( LogFileHandle );

   return;
}

/**
 * ==================================================================
 *
 * Function Name
 *  RemoveTrailingBlanks
 *
 * Description
 *  This routine will remove trailing blanks from a string.
 *
 * Input parameters
 *  pStr - a pointer to a string where trailing blanks are to be
 *           removed - if any.
 *
 * Output
 *  string - update string with trailing blanks removed.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void RemoveTrailingBlanks( char *pStr )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int   i;
   int   len = (int)strlen( pStr );  /* Length of String */

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   if (len <= 0)
      return; /* nothing to do */

   /*
    * Find the 1st blank beyond the last valid character.
    */
   for ( i = (len - 1 ); i >= 0 ; i-- )
   {
      if ( *(pStr + i) != ' ' )     /* look for a non-blank */
      {
         break;                 /* get out - we found a non-blank */
      }
   }

   /*
    * Set trailing blanks to nulls.
    */
   memset( pStr + i + 1, '\0', (len - i) );

   return;
}

/**
 * ==================================================================
 *
 * Function Name
 *  RemovePrefixBlanks
 *
 * Description
 *  This routine will remove any prefix blanks from a string.
 *
 * Input parameters
 *  string - a pointer to a string where prefix blanks are to be
 *           removed - if any.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void RemovePrefixBlanks( char *String )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int           Length;           /* Length of String */

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   /**
    * Find the 1st nonblank character.
    */
   for ( Length = 0; Length <= (int) strlen( String ); Length++ )
   {
       if ( *(String + Length) != ' ' )   /* look for a non-blank */
       {
          break;                 /* get out - we found a non-blank */
       }
   }

   /**
    * Remove beginning blanks.
    */
   if (Length != 0)
   {
      memcpy( String, String + Length, (strlen(String) - Length ) );
      memset( String + (strlen(String) - Length ), '\0', Length );
   }

   return;
}

/**
 * ==================================================================
 *
 * Function Name
 *  ConnectToMqm
 *
 * Description
 *  This routine will establish a connection between
 *  us and the local MQM. If it fails, put out a message
 *  and return a non-zero return code.
 *
 * Input parameters
 *  MG.QMgrName - Queue Manager name
 *
 * Output
 *  MG.Hconn - Connection handle
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  ConnectToMqm( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int       rc=0;
   MQLONG    mqCC;   /* return code from MQM */
   MQLONG    mqRC;   /* reason code from MQM */
   char      Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering ConnectToMqm\n")

   MQCONN( MG.QMgrName,         /* queue manager */
           &(MG.Hconn),         /* connection handle */
           &mqCC,               /* completion code */
           &mqRC);              /* reason code */

   rc = mqRC;

   memcpy( Buffer, MG.QMgrName, MQ_Q_MGR_NAME_LENGTH );

   Buffer[MQ_Q_MGR_NAME_LENGTH] = '\0';  /* make sure we have a null */
   RemoveTrailingBlanks( Buffer );

   sprintf( MG.TempBuffer,
            "MQConn CC=%d RC=%d on connection to %s",
            mqCC,
            mqRC,
            Buffer );

   if ( mqCC == MQCC_OK )
   {
      MG.ConnectedFlag =  1;

      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MG.TempBuffer );
   }
   else
   {
      MG.ConnectedFlag =  0;

      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MG.TempBuffer );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  DisconnectFromMqm
 *
 * Description
 *  This routine will sever an existing connection between
 *  us and MQM. If it fails, put out a message and return
 *  a non-zero return code.
 *
 * Input parameters
 *  MG.Hconn - Connection handle
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  DisconnectFromMqm( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int       rc=0;
   MQLONG    mqCC;   /* return code from MQM */
   MQLONG    mqRC;   /* reason code from MQM */
   char      Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering DisconnectFromMqm\n")

   MQDISC( &MG.Hconn,        /* connection handle */
           &mqCC,            /* completion code */
           &mqRC);           /* reason code */

   rc = mqRC;

   MG.ConnectedFlag =  0;

   memcpy( Buffer, MG.QMgrName, MQ_Q_MGR_NAME_LENGTH );

   Buffer[MQ_Q_MGR_NAME_LENGTH] = '\0';  /* make sure we have a null */
   RemoveTrailingBlanks( Buffer );

   sprintf( MG.TempBuffer,
            "MQDisc  CC=%d RC=%d on disconnecting from %s",
            mqCC,
            mqRC,
            Buffer );

   if ( mqCC == MQCC_OK )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MG.TempBuffer );
   }
   else
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MG.TempBuffer );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  OpenInputQueue
 *
 * Description
 *  This routine will open the input queue. If it fails,
 *  put out a message and return a non-zero return code.
 *
 * Input parameters
 *  MG.Hconn      - Connection handle for the local Queue Manager
 *  MG.InputQName - Input Q name
 *
 * Output
 *  MG.HobjInQ - Object handle for Input Q
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  OpenInputQueue( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int       rc=0;
   MQLONG    mqCC;       /* return code from MQM */
   MQLONG    mqRC;       /* reason code from MQM */
   MQOD      od  = {MQOD_DEFAULT};
   MQLONG    OpenOptions;       /* Open Options */
   char      Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering OpenInputQueue\n")

   memcpy( od.ObjectQMgrName,
           MG.QMgrName,
           MQ_Q_MGR_NAME_LENGTH);

   memcpy( od.ObjectName,
           MG.InputQName,
           MQ_Q_NAME_LENGTH);

   OpenOptions = MQOO_FAIL_IF_QUIESCING | MQOO_INPUT_EXCLUSIVE | MQOO_SAVE_ALL_CONTEXT;

   MQOPEN( MG.Hconn,              /* connection handle */
           &od,                   /* object descriptor for queue */
           OpenOptions,           /* open options */
           &(MG.HobjInQ),         /* object handle */
           &mqCC,                 /* completion code */
           &mqRC);                /* reason code */

   rc = mqRC;

  /* Output the Completion Code and Reason Code */

   memcpy( Buffer, MG.InputQName, sizeof(MG.InputQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';
   RemoveTrailingBlanks( Buffer );

   sprintf( MG.TempBuffer,
            "MQOpen CC=%d RC=%d for %s",
            mqCC,
            mqRC,
            Buffer);

   if ( mqCC == MQCC_OK )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MG.TempBuffer );

      MG.OpenedInputQueue = 1;
   }
   else
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MG.TempBuffer );

      MG.OpenedInputQueue = 0;
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  OpenOutputQueue
 *
 * Description
 *  This routine will open the output queue. If it fails,
 *  put out a message and return a non-zero return code.
 *
 * Input parameters
 *  None.
 *
 * Output
 *  MG.HobjOutQ - Object handle for Output Q
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  OpenOutputQueue( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int       rc=0;
   MQLONG    mqCC;       /* return code from MQM */
   MQLONG    mqRC;       /* reason code from MQM */
   MQOD      od  = {MQOD_DEFAULT};
   MQLONG    OpenOptions;       /* Open Options */
   char      Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering OpenOutputQueue\n")

   memcpy( od.ObjectQMgrName,
           MG.QMgrName,
           MQ_Q_MGR_NAME_LENGTH);

   memcpy( od.ObjectName,
           MG.OutputQName,
           MQ_Q_NAME_LENGTH);

   OpenOptions = MQOO_FAIL_IF_QUIESCING | MQOO_OUTPUT | MQOO_PASS_ALL_CONTEXT;

   MQOPEN( MG.Hconn,           /* connection handle */
           &od,                /* object descriptor for queue */
           OpenOptions,        /* open options */
           &(MG.HobjOutQ),   /* object handle */
           &mqCC,              /* completion code */
           &mqRC);             /* reason code */

   rc = mqRC;

  /* Output the Completion Code and Reason Code */

   memcpy( Buffer, MG.OutputQName, MQ_Q_NAME_LENGTH );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';
   RemoveTrailingBlanks( Buffer );

   sprintf( MG.TempBuffer,
            "MQOpen CC=%d RC=%d for %s",
            mqCC,
            mqRC,
            Buffer );

   if ( mqCC == MQCC_OK )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MG.TempBuffer );

      MG.OpenedOutputQueue = 1;
   }
   else
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MG.TempBuffer );

      MG.OpenedOutputQueue = 0;
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  CloseInputQueue
 *
 * Description
 *  This routine will close the input queue. If it fails,
 *  put out a message and return a non-zero return code.
 *
 * Input parameters
 *  MG.Hconn   - Connection handle for the local Queue Manager
 *  MG.HobjInQ - Object handle for Input Q
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  CloseInputQueue( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int       rc=0;
   MQLONG    mqCC;       /* return code from MQM */
   MQLONG    mqRC;       /* reason code from MQM */
   char      Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering CloseInputQueue\n")

   MQCLOSE( MG.Hconn,
            &(MG.HobjInQ),
            MQCO_NONE,
            &mqCC,
            &mqRC);

   MG.OpenedInputQueue = 0;

  /* Output the Completion Code and Reason Code */

   memcpy( Buffer, MG.InputQName, sizeof(MG.InputQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';
   RemoveTrailingBlanks( Buffer );

   sprintf( MG.TempBuffer,
            "MQClose CC=%d RC=%d for %s",
            mqCC,
            mqRC,
            Buffer);

   if ( mqCC == MQCC_OK )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MG.TempBuffer );
   }
   else
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MG.TempBuffer );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  CloseOutputQueue
 *
 * Description
 *  This routine will close the output queue. If it fails,
 *  put out a message and return a non-zero return code.
 *
 * Input parameters
 *  MG.Hconn    - Connection handle for the local Queue Manager
 *  MG.HobjOutQ - Object handle for Output Q
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  CloseOutputQueue( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int       rc=0;
   MQLONG    mqCC;       /* return code from MQM */
   MQLONG    mqRC;       /* reason code from MQM */
   char      Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering CloseOutputQueue\n")

   MQCLOSE( MG.Hconn,
            &(MG.HobjOutQ),
            MQCO_NONE,
            &mqCC,
            &mqRC);

   MG.OpenedOutputQueue = 0;

  /* Output the Completion Code and Reason Code */

   memcpy( Buffer,
           MG.OutputQName,
           sizeof(MG.OutputQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';

   RemoveTrailingBlanks( Buffer );

   sprintf( MG.TempBuffer,
            "MQClose CC=%d RC=%d for %s",
            mqCC,
            mqRC,
            Buffer );

   if ( mqCC == MQCC_OK )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MG.TempBuffer );
   }
   else
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MG.TempBuffer );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  ProcessMessages
 *
 * Description
 *  This routine will browse the Input Q for any messages that
 *  have the particaular named field with the value that matches
 *  what the user wants.
 *
 * Input parameters
 *  MG.Hconn    - connection handle
 *  MG.HobjInQ  - Object handle for Input Q
 *  MG.HobjOutQ - Object handle for Output Q
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  ProcessMessages( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int         rc=0;
   char        Continue   = 'Y';

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering ProcessMessages\n")

   MG.MessageBuffer = malloc( MG.MessageBufferSize + 1 );

   memset( MG.MessageBuffer, '\0', MG.MessageBufferSize );

   /**
    * Setup and get first message.
    */
   rc = ReadMessage( );

   if ( rc != MQRC_NONE )
      Continue = 'N';

   /**
    * Loop through the messages on the queue.
    */
   while ( (Continue == 'Y') && (rc == MQRC_NONE) )
   {
      rc = PutMessage();

      if ( rc == MQRC_NONE )
         rc = CommitMessages();
      else
         BackOutMessages();

      if ( rc == MQRC_NONE )
         rc = ReadMessage();
      else
         Continue = 'N';
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  ReadMessage
 *
 * Description
 *  This routine will read (either browse or destructive get based on
 *  extra get options) a message from a queue.  If it fails, put
 *  out a message and return a non-zero return code.
 *
 * Input parameters
 *  ExtraOptions  - Extra get options either browse or destructive get
 *  MG.Hconn    - Connection handle for the local Queue Manager
 *  MG.HobjOutQ - Object handle for Output Q
 *
 * Output
 *  MG.MessageBuffer - data returned from MQGET
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  ReadMessage( )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int           rc=0;
   MQLONG        mqCC;       /* return code from MQM */
   MQLONG        mqRC;       /* reason code from MQM */
   MQLONG        MessageLength = 0;
   MQGMO         gmo = {MQGMO_DEFAULT};
   MQMD          md  = {MQMD_DEFAULT};
   char          Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering ReadMessage\n")

   gmo.Options = MQGMO_FAIL_IF_QUIESCING | MQGMO_WAIT | MQGMO_SYNCPOINT;

   gmo.WaitInterval = MQWI_UNLIMITED;

   memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
   memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

   MQGET( MG.Hconn,                 /* connection handle */
          MG.HobjInQ,               /* object handle */
          &md,                      /* message descriptor */
          &gmo,                     /* get message options */
          MG.MessageBufferSize,     /* buffer length */
          MG.MessageBuffer,         /* message buffer */
          &MG.ActualMessageSize,    /* returned message length */
          &mqCC,                    /* completion code */
          &mqRC);                   /* reason code */

   rc = mqRC;

   /* Save the MQMD too. */
   memcpy( &(MG.OriginalMD), &(md), sizeof(MQMD) );

   /* Output the Completion Code and Reason Code */
   memcpy( Buffer,
           MG.InputQName,
           sizeof(MG.InputQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';

   RemoveTrailingBlanks( Buffer );

   sprintf( MG.TempBuffer,
            "MQGet  CC=%d RC=%d for %s",
            mqCC,
            mqRC,
            Buffer);

   /**
    * Did everything work?? i.e. returned a message
    */
   if ( (mqCC == MQCC_OK) && (mqRC == MQRC_NONE) )
   {

      if (md.Feedback == MQFB_QUIT)
      {
         LogIt( LOG_TYPE_INFO,
                __LINE__,
                "Quit message received - time to go." );

         rc = CommitMessages();

         rc = 5;
      }

      MG.TotalMsgRead++;
   }

   /**
    * No more messages??
    */
   else if ( (mqCC != MQCC_OK) &&
             (mqRC == MQRC_NO_MSG_AVAILABLE) )
   {
      /* No more messages on the queue. */

      LogIt( LOG_TYPE_WARNING,
             __LINE__,
             MG.TempBuffer );
   }

   else if ( (mqCC == MQCC_FAILED) &&
             ( (mqRC == MQRC_Q_MGR_QUIESCING)     ||
               (mqRC == MQRC_Q_MGR_NOT_ACTIVE)    ||
               (mqRC == MQRC_Q_MGR_NOT_AVAILABLE) ||
               (mqRC == MQRC_Q_MGR_STOPPING)       ) )
   {
      LogIt( LOG_TYPE_WARNING,
             __LINE__,
             MG.TempBuffer );

      LogIt( LOG_TYPE_WARNING,
             __LINE__,
             "Queue manager is no longer available" );
   }

   else if ( (mqCC == MQCC_WARNING) &&
             (mqRC == MQRC_FORMAT_ERROR) )
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MG.TempBuffer );

      LogIt( LOG_TYPE_WARNING,
             __LINE__,
             "Use GetWithConvert=N to avoid this error." );

      BackOutMessages();
   }

   /**
    * Something catastrophic happen!!
    */
   else if ( mqCC != MQCC_OK )
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MG.TempBuffer );

      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             "*** Severe Error accessing the queue. ***" );

      BackOutMessages();
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  PutMessage
 *
 * Description
 *  This routine will write (put) a message to the output queue.
 *  If it fails, put out a message and return a non-zero return code.
 *
 * Input parameters
 *  MG.Hconn    - Connection handle for the local Queue Manager
 *  MG.HobjOutQ - Object handle for Output Q
 *  MG.MessageBuffer - outbound message buffer.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  PutMessage( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int          rc=0;
   MQLONG       mqCC;       /* return code from MQM */
   MQLONG       mqRC;       /* reason code from MQM */
   int          MessageLength = 0;
   MQPMO        pmo = {MQPMO_DEFAULT};
   MQMD         md  = {MQMD_DEFAULT};
   char         Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering PutMessage\n")

   /* move original MD values. */
   memcpy( &(md), &(MG.OriginalMD), sizeof(MQMD) );

   /**
    * Set MQPUT options
    */
   pmo.Options = MQPMO_SYNCPOINT | MQPMO_PASS_ALL_CONTEXT | MQPMO_FAIL_IF_QUIESCING;

   /* pass in the Input Queue's handle */
   pmo.Context = MG.HobjInQ;

   MQPUT( MG.Hconn,              /* connection handle */
          MG.HobjOutQ,           /* object handle */
          &md,                   /* message descriptor */
          &pmo,                  /* put message options */
          MG.ActualMessageSize,  /* buffer length */
          MG.MessageBuffer,      /* message buffer */
          &mqCC,                 /* completion code */
          &mqRC);                /* reason code */

   if (mqRC != MQRC_NONE)
      rc = mqRC;

   /* Output the Completion Code and Reason Code */
   memcpy( Buffer,
           MG.OutputQName,
           sizeof(MG.OutputQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';

   RemoveTrailingBlanks( Buffer );

   sprintf( MG.TempBuffer,
            "MQPut  CC=%d RC=%d for %s",
            mqCC,
            mqRC,
            Buffer );

   /**
    * Was the put successful??
    */
   if ( (mqCC == MQCC_OK) && (mqRC == MQRC_NONE) )
   {
      /* Life's good!!!! */
      MG.TotalMsgWritten++;
   }

   else if ( (mqCC == MQCC_FAILED) &&
             ( (mqRC == MQRC_Q_MGR_QUIESCING)     ||
               (mqRC == MQRC_Q_MGR_NOT_ACTIVE)    ||
               (mqRC == MQRC_Q_MGR_NOT_AVAILABLE) ||
               (mqRC == MQRC_Q_MGR_STOPPING)       ) )
   {
      LogIt( LOG_TYPE_WARNING,
             __LINE__,
             MG.TempBuffer );

      LogIt( LOG_TYPE_WARNING,
             __LINE__,
             "Queue manager is no longer available" );
   }

   /**
    * Something catastrophic happen!! Again!!
    */
   else if ( mqCC != MQCC_OK )
   {
      LogIt( LOG_TYPE_ERROR, __LINE__, MG.TempBuffer );

      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             "*** Severe Error accessing the queue. ***" );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  CommitMessages
 *
 * Description
 *  This routine will commit the previous gets and puts.
 *
 * Input parameters
 *  MG.Hconn    - Connection handle for the local Queue Manager
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  CommitMessages( )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int           rc=0;
   MQLONG        mqCC;       /* return code from MQM */
   MQLONG        mqRC;       /* reason code from MQM */

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering CommitMessages\n")

   MQCMIT( MG.Hconn,                 /* connection handle */
           &mqCC,                    /* completion code */
           &mqRC);                   /* reason code */

   rc = mqRC;

   sprintf( MG.TempBuffer,
            "MQCmit CC=%d RC=%d",
            mqCC,
            mqRC );

   /**
    * Did everything work?? i.e. returned a message
    */
   if ( (mqCC == MQCC_OK) && (mqRC == MQRC_NONE) )
   {
   }

   /**
    * Something catastrophic happen!!
    */
   else if ( mqCC != MQCC_OK )
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MG.TempBuffer );

      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             "*** Severe Error committing the messages. ***" );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  BackOutMessages
 *
 * Description
 *  This routine will back out the message.
 *
 * Input parameters
 *  MG.Hconn    - Connection handle for the local Queue Manager
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  BackOutMessages( )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int           rc=0;
   MQLONG        mqCC;       /* return code from MQM */
   MQLONG        mqRC;       /* reason code from MQM */

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering BackOutMessages\n")

   MQBACK( MG.Hconn,                 /* connection handle */
           &mqCC,                    /* completion code */
           &mqRC);                   /* reason code */

   rc = mqRC;

   sprintf( MG.TempBuffer,
            "MQBack CC=%d RC=%d",
            mqCC,
            mqRC );

   /**
    * Did everything work?? i.e. returned a message
    */
   if ( (mqCC == MQCC_OK) && (mqRC == MQRC_NONE) )
   {
   }

   /**
    * Something catastrophic happen!!
    */
   else if ( mqCC != MQCC_OK )
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MG.TempBuffer );

      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             "*** Severe Error committing the messages. ***" );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  ReadLine
 *
 * Description
 *  This function will read a line from the specified file.  If the
 *  sequence number flag is on then truncate the line at 72 characters.
 *
 * Input parameters
 *  fp - a pointer to the file to be read from
 *  bp - a pointer to the copy buffer
 *  SeqNumFlag - MVS sequence ## in the file?
 *
 * Output
 *  None.
 *
 * Return Value
 *  TRUE if successful FALSE otherwise
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  ReadLine( FILE     *fp,
               char     *bp,
               char     SeqNumFlag )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
    char    c = '\0';
    int     i = 0;

    /* --------------------------------------------
     * Code section
     * --------------------------------------------
     */
/*   TEST_PRINT("Now entering ReadLine\n") */

   /**
    * Read one line at a time from the source file
    */
   if (!feof (fp) && fgets (bp, MAX_INI_LINE_LENGTH, fp) != 0)
   {
      if ( strchr (bp, '\r') )
      {
          *strchr (bp, '\r') = '\0';
      }
      else if ( strchr (bp, '\n') )
      {
          *strchr (bp, '\n') = '\0';
      }
      else
      {
         /**
          * Make sure the line is at a length that we can handle.
          */
         memset( (bp+MAX_INI_LINE_LENGTH-1), 0 , 1 );
      }

      /**
       * If the sequence number flag is set then truncate the line
       * at column 72.
       */
      if ( (SeqNumFlag == 'y') || (SeqNumFlag == 'Y') )
      {
         bp[71] = '\0';   /* sequence numbers begin in column 72 */
      }

      return(1);   /* Yes!! we have a line. */
   }

   return(0);   /* !! no line. */
}

/**
 * ==================================================================
 *
 * Function Name
 *  GetProfileInt
 *
 * Description
 *  This function will retrieve an integer from an initialization
 *  file based on the provided field "entry" name.
 *
 * Input parameters
 *  entry - the name of the entry to find the value of
 *  def - the default value in the event of a failed read
 *  file_name - the name of the .ini file to read from
 *  SeqNumFlag - MVS sequence ## in the file?
 *
 * Output
 *  None.
 *
 * Return Value
 *  the value located at entry
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int GetProfileInt( char   *entry,
                   int     def,
                   char   *file_name,
                   char    SeqNumFlag)
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   FILE           *fp;
   int             len;
   int             i;
   char           *ep;
   char            buff[MAX_INI_LINE_LENGTH] = {'\0'};
   char            value[12];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering GetProfileInt\n")

   memset( buff, '\0', MAX_INI_LINE_LENGTH );

   len = strlen(entry);

   /**
    * Open up the ini file.
    */
   if ( !(fp = fopen(file_name,"r")) )
   {
      return( def );
   }

   /**
    * Move through file 1 line at a time until a the entry
    * is matched or EOF. '[' is section header - end if found.
    */
   do
   {
       if( !ReadLine( fp, buff, SeqNumFlag ) ||
            buff[0] == '\0' ||
            buff[0] == '[')
       {
           fclose( fp );
           return( def );
       }
   }  while( strncmp( buff, entry, len) );

   ep = strrchr(buff,'=');    /* Parse out the equal sign */
   ep++;

   if( !strlen(ep) )          /* No setting? */
   {
       return(def);
   }

   /**
    * Remove unnecessary blanks
    */
   RemovePrefixBlanks( ep );
   RemoveTrailingBlanks( ep );

   /**
    * Copy only numbers - fail on characters
    */
   for(i = 0; isdigit(ep[i]); i++ )
   {
       value[i] = ep[i];
   }

   value[i] = '\0';
   fclose(fp);                /* Clean up and return the value */

   return( atoi(value) );
}

/**
 * ==================================================================
 *
 * Function Name
 *  GetProfileString
 *
 * Description
 *  This function will retrieve a string from an initialization
 *  file based on the provided field "entry" name.
 *
 * Input parameters
 *  section - the name of the section to search for
 *  entry - the name of the entry to find the value of
 *  def - default string in the event of a failed read
 *  buffer - a pointer to the buffer to copy into
 *  buffer_len - the max number of characters to copy
 *  file_name - the name of the .ini file to read from
 *  SeqNumFlag - MVS sequence ## in the file?
 *
 * Output
 *  None.
 *
 * Return Value
 *  number of characters copied into the supplied buffer
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int GetProfileString( char   *entry,
                      char   *def,
                      char   *buffer,
                      int     buffer_len,
                      char   *file_name,
                      char    SeqNumFlag)
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   FILE           *fp;
   int             len = strlen(entry);
   char           *ep;
   char            buff[MAX_INI_LINE_LENGTH] = {'\0'};

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering GetProfileString\n")

   memset( buff, '\0', MAX_INI_LINE_LENGTH );

   /**
    * Open up the ini file.
    */
   if ( !(fp = fopen(file_name,"r")) )
   {
        strncpy(buffer,def,buffer_len);
        return(strlen(buffer));
   }

   /**
    * Move through file 1 line at a time until a the entry
    * is matched or EOF. '[' is section header - end if found.
    */
   do
   {
      if( !ReadLine(fp, buff, SeqNumFlag) ||
           buff[0] == '\0' ||
           buff[0] == '[' )
       {
           fclose(fp);
           strncpy(buffer,def,buffer_len);
           return(strlen(buffer));
       }
   }  while( strncmp(buff,entry,len) );

   ep = strrchr(buff,'=');    /* Parse out the equal sign */
   ep++;

   /**
    * Copy up to buffer_len chars to buffer
    */
   strncpy(buffer,ep,(buffer_len - 1) );

   /**
    * Remove unnecessary blanks
    */
   RemovePrefixBlanks( buffer );
   RemoveTrailingBlanks( buffer );

   /**
    * Clean up and return the amount copied
    */
   fclose(fp);

   return( strlen(buffer) );
}

/**
 * ==================================================================
 *
 * Function Name
 *  CalculateMemorySize
 *
 * Description
 *  This routine will calculate the required memory size. It
 *  rounds the number up to the nearest 1 KB boundary.
 *
 * Input parameters
 *  Length - Input length to be used in the base calculation.
 *
 * Output
 *  The length rounded to the nearest 4 KB boundary.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  CalculateMemorySize( int Length )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering CalculateMemorySize\n")

   return ( ((Length + 1023) / 1024) * 1024 );
}

/* End of file */
